/* dclock.c  C-COMPILER-51 Alarm Clock Demo
 * Copyright Franklin Software, Inc., 1988,1989
 *
 * This file is part of the Franklin Software C-51 compiler demonstration
 * package, and may be freely distributed, as long as the entire demo package
 * is kept intact.
 *
 * For information or assistance WRITE, CALL or FAX:
 *			Franklin Software, Inc.
 *			888 Saratoga Ave. Suite #2
 *			San Jose, Ca.  95129
 *			(408) 296-8051	for general information 
 *			(408) 296-8061	FAX
 *
 * FileName:	dclock.c, the Franklin Software C-51 Compiler Alarm Clock Demo
 * Purpose:	to illustrate the effectiveness of the Franklin C Compiler
 *
 * History:	871222 - swb - initial release
 *		890320 - swb - cleaned up and added this header
 */

#include <reg51.h>
#include <stdio.h>


/* define time characteristics (Clock FREQUENCY in MHz) */
#define FREQ    6

/****** If a real 8051 CPU is used, redefine value ******/
#ifdef CPU==8051
#define SECOND  (4000/(12/FREQ))
#else
#define SECOND  (200/(12/FREQ))
#endif
/********************************************************/

#define PERIOD  (-250)


/* global data declarations */
bit  alarm;                    /* alarm flag */
bit  ring;                     /* alarm ring flag */

sbit hwringer = P3^5;          /* Hardware-Port for ringer */

unsigned int  count;           /* 500usec counter */


/* data structures, for the timer routines */
struct time  {
    unsigned char   hour;      /* hour */
    unsigned char   min;       /* minute */
    unsigned char   sec;       /* second */
};

struct time clocktime;         /* current time */
struct time alarmtime;         /* alarm time */


/* strings for the user, first the heading string */
code char heading[] = "Franklin C-51 C Compiler Alarm Clock Demo\n\n\n";

/* then the menu string */
code char mnu[] = {
	"Function Selection\n\n" \
	"1 - Display Time\n"     \
	"2 - Change Time\n"      \
	"3 - Change Alarm\n"     \
	"4 - Set Alarm\n"        \
	"5 - Clear Alarm\n\n"    \
	"Enter Selection:  "
};


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Name:	PROCedure sendbell()
 * Purpose:	to send a bell signal
 * Input:	none
 * Function:	check to ensure that the serial interface is free, then beep
 * Output:	none
 *
 * Scope:	none
 */
sendbell ()
{
	if (TI)  {
		TI = 0;
		SBUF = '\a';
	}
}	/* end of sendbell() */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Name:	PROCedure timer0()
 * Purpose:	timer 0 interrupt service function
 * Input:	none, this is an interrupt trap
 * Function:	executes once each timer 0 interrupt, maintains the hours, 
 *		minutes and seconds counters.
 * Output:	none
 *
 * Scope:	sendbell()
 */
timer0() interrupt 1 using 2
{
	/* Second Interval Processor */
	if( ++count == SECOND ) {	/* increment interrupt counter */
		if( alarm			     &&
		    alarmtime.hour == clocktime.hour &&
		    alarmtime.min  == clocktime.min  &&
		    alarmtime.sec  == clocktime.sec     )
		        ring = 1;	/* set alarm ring flag */

		if( ring ) {
			hwringer = 1;	/* Activate Hardware Ringer */
			sendbell ();	/* send a BELL character */
		}
		else hwringer = 0;	/* disable Hardware Ringer */

		count = 0;

		if( ++clocktime.sec == 60 ) {	/* inc sec counter */
			clocktime.sec = 0;	/* Minute Interval Processor */
			sendbell ();

			if( ++clocktime.min == 60 ) {	/* inc min counter */
				clocktime.min = 0;	/* Hour Interval */

				if( ++clocktime.hour == 25 ) {
					clocktime.hour = 1;
				}	/* inc. or adjust the hour counter */
			}
		}	/* of if ++clocktime.sec ... */
	}	/* of if ++counter... */
}	/* end of PROCedure timer0() */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Name:	FUNCtion menu()
 * Purpose:	displays the alarm menu
 * Input:	none
 * Function:	selects and returns selected function character
 * Output:	the selected function value as an integer
 *
 * Scope:	printf(), _getkey();
 */
menu()
{
  printf (heading);                 /* print heading */
  printf (mnu);                     /* Menue */
  return (_getkey());               /* get character, no echo */
}	/* end of FUNCtion menu() */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Name:	PROCedure update()
 * Purpose:	updates the time and alarm display
 * Input:	a structure of the time elements
 * Function:	prints the time and alarm display
 * Output:	none
 *
 * Scope:	none
 */
update (dtime)
struct time *dtime;
{
  printf("\r   HH:MM:SS   %02bd:%02bd:%02bd  ALARM %s",
                          dtime->hour,
                          dtime->min,
                          dtime->sec,
                          alarm?(ring?"RINGING":"SET"):"CLEAR");
}	/* end of PROCedure update() */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Name:	PROCedure display()
 * Purpose:	to update the time display once per second
 * Input:	none
 * Function:	display the time, and exit on input of the <CR> key
 * Output:	none
 *
 * Scope:	printf(), _getkey(), update();
 */
display()
{
	unsigned char last = 0xFF;	/* last sec displayed */

	printf("%s",heading);		/* print heading */
	printf("\nCurrent Time, Type <CR> to exit\n\n\n");

	while (1)  {
		if (RI && (_getkey() == '\r'))	/* exit on <CR> */
			break;

		if (last != clocktime.sec)  {	/* check for second change */
			update(&clocktime);	/* update time display */
			last = clocktime.sec;
		}
	}	/* of while 1 ... */
}	/* end of PROCedure display() */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Name:	PROCedure setime()
 * Purpose:	used to set current time and alarm time
 * Input:	alarm mode, and time of day
 * Function:	display current time, and get new times, exit on <CR>
 * Output:	none
 *
 * Scope:	printf(), update(), _getkey();
 */
setime(char *mode, struct time *dtime)
{
	struct time setuptime;           /* setup-time */
	setuptime.hour = dtime->hour;    /* copy current timer/alarm */
	setuptime.min  = dtime->min;
	setuptime.sec  = dtime->sec;

	printf ("\n\n%sChange %s"         \
		"\n\nKey     Function"    \
		"\n-------------------"   \
		"\n 1      Change hour"   \
		"\n 2      Change min"    \
		"\n 3      Change sec"    \
		"\n <cr>   Set time"      \
		"\n <ESC>  Quit\n\n"      \
		,heading,mode);          /* print heading */

	update(&setuptime);             /* update time display */
	RI = 0;                         /* clear serial port receiver flag */

	while (1)  {
		switch (_getkey())  {         /* get character, no echo */

			case '\r':                  /* set time on <CR> */
				dtime->hour = setuptime.hour;
				dtime->min  = setuptime.min;
				dtime->sec  = setuptime.sec;
						/* fall through into ESC */

			case 0x1B:  return;	/* Quit on <ESC> */

			case '1':                   /* change hour on '1' */
			        if (++setuptime.hour == 25)  setuptime.hour = 1;
			        break;

			case '2':                   /* change min */
				if (++setuptime.min == 60)  setuptime.min = 0;
				break;

			case '3':                   /* change sec */
				if (++setuptime.sec == 60)  setuptime.sec = 0;
				break;

			default:  continue;

		}	/* of switch on _getkey() */

	update(&setuptime);           /* update time/alarm display */
	}	/* end of while( 1 ) ... */
}	/* end of PROCedure setime() */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Name:	PROCedure main()
 * Purpose:	to provide the execution and drive control for this program
 * Input:	none
 * Function:	program main
 * Output:	none
 *
 * Scope:	
 */
void main()
{
	/* init variables */
	alarmtime.hour = clocktime.hour = 12;
	alarmtime.min = alarmtime.sec = clocktime.min = clocktime.sec = 0;
	alarm = 0;
	ring = 0;

/* setup serial port control hardware (1200 BAUD @6MHZ) */

	SCON = 0x52;    /* SCON */
	TMOD = 0x20;    /* TMOD */
	TCON = 0x69;    /* TCON */
	TH1 =  0xf3;    /* TH1 */

/* setup timer 0 interrupt */
	TH0 = PERIOD;            /* set timer period */
	TL0 = PERIOD;
	TMOD = TMOD | 0x02;      /* select mode 2 */
	TR0 = 1;                 /* start timer 0 */
	ET0 = 1;                 /* enable timer 0 interrupt */
	EA = 1;                  /* global interrupt enable */

	hwringer = 0;            /* clear hardware ringer */

/* program main routine, do forever */
	while (1)  {
		switch (menu())  {    /* timer menu */
			case '1': display();  break;	
			case '2': setime("Time",&clocktime); break;
			case '3': setime("Alarm",&alarmtime); break;
			case '4': alarm = 1; break;
			/* clear alarm and ring */
			case '5': alarm = ring = 0;  break; 
		}	/* end of switch on menu response ... */
	}	/* end of while loop ... */
}	/* end of main() */

/* 
 * Franklin Software, Inc. 888 Saratoga Ave. Suite #2, San Jose, Ca.  95129
 * (408) 296-8051 for help or general information, (408) 296-8061 for FAX
 */
